<?php declare(strict_types=1);

namespace Micropayment\Helper;

if (!defined('MCPAY_BASE_PATH')) {
    define('MCPAY_BASE_PATH', dirname(__FILE__).'/../../vendor/mcpay/');
}
require_once(MCPAY_BASE_PATH.'classes/class.mcpay.php');

use \mcpay as mcpay;

class McPayment extends mcpay
{
    var $cfg;

    /**
     * McPayment constructor.
     */
    public function __construct()
    {
    }

    /**
     * init
     *
     * @param $config
     *
     * @return array
     */
    public function init($config)
    {
        $testMode = $config['McPayDEBUG'];
        $debugLog = NULL;
        if (!empty($config['McPayDEBUGLOG'])) {
            $debugLog = $config['McPayDEBUGLOG'];
        }
        parent::__construct($testMode, $debugLog);

        $cfg = $this->getCfg($config);
        $this->cfg = $cfg;
        return $cfg;
    }

    /**
     * getCfg
     *
     * @param $config
     *
     * @return array
     */
    private function getCfg($config)
    {
        // key and value tauschen
        $configMap = array(
            'McPayACCESSKEY'    => 'accessKey',
            'McPayBGCOLOR'      => 'bgcolor',
            'McPayBGGFX'        => 'bggfx',
            'McPayCCARDFORMID'  => '',
            'McPayDEBUG'        => 'debug',
            'McPayDEBUG_LOG'    => '',
            'McPayDOMAIN'       => 'suffix',
            'McPayGFX'          => 'gfx',
            'McPayPAYFORMID'    => '',
            'McPayPROJECTID'    => 'project',
            'McPaySECRETFIELD'  => 'secretfield',
            'McPaySEPAFORMID'   => '',
            'McPayTESTMODE'     => 'testMode',
            'McPayTHEME'        => 'theme',
            'McPayJSURL'        => 'plugin_urljs',
            'McPayCSSURL'       => 'plugin_urlcss',
            'McPaySTART3D'      => 'start3D',
            'McPaySTART3DREUSE' => 'start3Dreuse',
            'McPayEXPIREDAYS'   => 'expiredays',
            'McPayFREETEXT'     => 'paytext',
        );
        $cfg       = array();
        foreach ($configMap as $k => $v) {
            if (empty($v)) continue;
            if (empty($config[$k])) {
                $cfg[$v] = '';
            } else {
                $cfg[$v] = $config[$k];
            }
        }
        //$cfg['paytext'] = 'Your Order: {$orderid}';
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$cfg', print_r($cfg,1).print_r($config,1));
        $this->setConfig($cfg);

        return $cfg;
    }

    /**
     * getUserData
     *
     * @param \Shopware\Core\Checkout\Order\OrderEntity $order
     * @param $config
     * @param \Shopware\Core\System\SalesChannel\SalesChannelContext $salesChannelContext
     *
     * @return array
     */
    function getUserData($order, $config, $salesChannelContext)
    {
        $user = $order->getOrderCustomer();
        $customer = $salesChannelContext->getCustomer();

        $orderId = $order->getOrderNumber();
        $custID = $user->getCustomerNumber();

        $customerBillingAddress = $customer->getActiveBillingAddress();
        if ($customerBillingAddress === null){
            $customerBillingAddress = $customer->getDefaultBillingAddress();
        }

        $amount = $order->getAmountTotal();
        $currency = 'EUR';
        $currencyObj = $order->getCurrency();
        if ($currencyObj === null) {
            $currencyObj = $salesChannelContext->getCurrency();
        }
        if ($currencyObj !== null){
            $currency = $currencyObj->getIsoCode();
        }
        $countryISO = $customerBillingAddress->getCountry()->getIso();

//        $infos       = $mpBootstrap->getInfo();
        $chksumData = array(
            'amount'        => $amount * 100,
            'currency'      => $currency,
            'iso'           => $countryISO,
            'host'          => $_SERVER['HTTP_HOST'],
            'project'       => $this->project,
            'shoptype'      => 'shopware',
            'shopversion'   => '6.X',//Shopware()->Config()->version,
            'plugintype'    => 'WEB',
            'pluginversion' => '1.0.0',//(string)$infos['version'],
            'pm'            => $this->paymethod,
        );

        // take browser language
        $lang = strtolower(substr($_SERVER['HTTP_ACCEPT_LANGUAGE'],0,2));
        if(!in_array($lang, array('de', 'en'))){
            $lang = 'de';
        }

        $userData = array(
            'company'    => $customer->getCompany(),
            'firstName'  => $customer->getFirstName(),
            'lastName'   => $customer->getLastName(),
            'salutation' => $user->getSalutation()->getDisplayName(),
            'street'     => $customerBillingAddress->getStreet(),
            'zip'        => $customerBillingAddress->getZipcode(),
            'city'       => $customerBillingAddress->getCity(),
            'country'    => $countryISO,
            'email'      => $customer->getEmail(),
            'ip'         => $_SERVER['REMOTE_ADDR'],
            'id'         => $custID,
            'order_id'   => $orderId,
            'lang'       => $lang, //$order->getLanguage()->getName(), // sadly always empty
            'amount'     => ($amount * 100),
            'currency'   => $currency,
            'chksum'     => $chksumData, // request security
            'doFinish'   => false,
            'guest'      => (bool)($customer->getGuest() == 1),
        );
        if (empty($userData['ip'])) $userData['ip'] = $_SERVER['REMOTE_ADDR']; // Falls IP Leer, dann aus dem Server holen

        return $userData;
    }

    /**
     * getURL
     *
     * @param $userData
     * @param $activePayment
     *
     * @return string
     * @throws \Exception
     */
    public function getURL($userData, $activePayment)
    {
        $config = $this->cfg;
        $config['extraParams']['pm'] = $activePayment;
        $forwardURL = $this->getPaymentWindowParams($userData, $config, $activePayment);
        return $forwardURL;
    }
}
