<?php declare(strict_types=1);

namespace Micropayment\Helper;

use Micropayment\Service\Method\Call2payWeb;
use Micropayment\Service\Method\CreditcardWeb;
use Micropayment\Service\Method\HandypayWeb;
use Micropayment\Service\Method\PaydirektWeb;
use Micropayment\Service\Method\PaypalWeb;
use Micropayment\Service\Method\PaysafecardWeb;
use Micropayment\Service\Method\PrepayWeb;
use Micropayment\Service\Method\SepaWeb;
use Micropayment\Service\Method\SofortWeb;
use Micropayment\Service\Method\CreditcardApi;
use Micropayment\Service\Method\SepaApi;
use Micropayment\Service\Method\PrepayApi;
use Shopware\Core\Content\Media\MediaCollection;
use Shopware\Core\Content\Media\MediaService;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Symfony\Component\DependencyInjection\ContainerInterface;

class MediaHelper
{
    /** @var ContainerInterface */
    private $container;

    /** @var MediaService */
    private $mediaService;

    /** @var EntityRepository */
    private $mediaRepository;

    /**
     * MediaHelper constructor.
     *
     * @param                                                                         $container
     * @param \Shopware\Core\Content\Media\MediaService                               $mediaService
     * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository          $mediaRepository
     */
    public function __construct(
        $container,
        MediaService $mediaService,
        EntityRepository $mediaRepository
    )
    {
        $this->container       = $container;
        $this->mediaService    = $mediaService;
        $this->mediaRepository = $mediaRepository;
    }

    /**
     * getMediaId
     *
     * @param string                           $paymentMethod
     * @param \Shopware\Core\Framework\Context $context
     *
     * @return string
     */
    public function getMediaId(string $paymentMethod, Context $context, $lang = 'de'): string
    {
        /** @var string $mediaId */
        $mediaId = '';

        $mediaURLs = array(
          CreditcardWeb::class  => 'https://www.micropayment.de/resources/?what=img&group=cc&show=type-o.1&lang='.$lang,
          SepaWeb::class        => 'https://www.micropayment.de/resources/?what=img&group=dbt&show=type-o.1&lang='.$lang,
          SofortWeb::class      => 'https://www.micropayment.de/resources/?what=img&group=klarna-sofort&show=type-o.1&lang='.$lang,
          PaypalWeb::class      => 'https://www.micropayment.de/resources/?what=img&group=ppl&show=type-o.1&lang='.$lang,
          PrepayWeb::class      => 'https://www.micropayment.de/resources/?what=img&group=pp&show=type-o.1&lang='.$lang,
          PaysafecardWeb::class => 'https://www.micropayment.de/resources/?what=img&group=psc&show=type-o.1&lang='.$lang,
          PaydirektWeb::class   => 'https://www.micropayment.de/resources/?what=img&group=pdk&show=type-o.1&lang='.$lang,
          Call2payWeb::class    => 'https://www.micropayment.de/resources/?what=img&group=c2p&show=type-o.1&lang='.$lang,
          HandypayWeb::class    => 'https://www.micropayment.de/resources/?what=img&group=hp&show=type-o.1&lang='.$lang,
          CreditcardApi::class  => 'https://www.micropayment.de/resources/?what=img&group=cc&show=type-o.1&lang='.$lang,
            SepaApi::class        => 'https://www.micropayment.de/resources/?what=img&group=dbt&show=type-o.1&lang='.$lang,
          PrepayApi::class      => 'https://www.micropayment.de/resources/?what=img&group=pp&show=type-o.1&lang='.$lang,
        );

        $tmp = str_replace('Micropayment\\Service\Method\\', '', $paymentMethod);
        /** @var string $fileName */
        $fileName = $tmp.'-icon';

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('fileName', $fileName));

        /** @var MediaCollection $icons */
        $icons = $this->mediaRepository->search($criteria, $context);

        if ($icons->count() && $icons->first() !== NULL) {
            $mediaId = $icons->first()->getId();
        } else {
            // Add icon to the media library
            $iconBlob = file_get_contents($mediaURLs[$paymentMethod]);

            $mediaId = $this->mediaService->saveFile(
                $iconBlob,
                'png',
                'image/png',
                $fileName,
                $context,
                'McPayIcons',
                NULL,
                FALSE
            );
        }

        return $mediaId;
    }
}
