<?php declare(strict_types=1);

namespace Micropayment;

use Micropayment\Helper\CustomFieldHelper;
use Micropayment\Helper\MediaHelper;
use Micropayment\Service\Method\CreditcardWeb;
use Micropayment\Service\Method\SepaWeb;
use Micropayment\Service\Method\SofortWeb;
use Micropayment\Service\Method\PaypalWeb;
use Micropayment\Service\Method\PaysafecardWeb;
use Micropayment\Service\Method\PaydirektWeb;
use Micropayment\Service\Method\PrepayWeb;
//use Micropayment\Service\Method\Call2payWeb;
//use Micropayment\Service\Method\HandypayWeb;
use Micropayment\Service\Method\CreditcardApi;
use Micropayment\Service\Method\SepaApi;
use Micropayment\Service\Method\PrepayApi;

use Shopware\Core\Framework\Context;
//use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Plugin;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\Framework\Plugin\Context\UpdateContext;
use Shopware\Core\Framework\Plugin\Util\PluginIdProvider;

class Micropayment extends Plugin
{

    /**
     * install
     *
     * @param \Shopware\Core\Framework\Plugin\Context\InstallContext $context
     */
    public function install(InstallContext $context): void
    {
        $this->addPaymentMethod($context->getContext());

        // Add custom fields
        $customFieldHelper = new CustomFieldHelper(
            $this->container,
            $this->container->get('custom_field_set.repository')
        );

        $customFieldHelper->addCustomFields($context->getContext());
    }

    public function update(UpdateContext $context): void
    {
      $this->addPaymentMethod($context->getContext());
    }

    /**
     * uninstall
     *
     * @param \Shopware\Core\Framework\Plugin\Context\UninstallContext $context
     */
    public function uninstall(UninstallContext $context): void
    {
      //$this->setPaymentMethodIsActive(TRUE, $context->getContext()); // activate to uninstall because services wont work deactivated
      // Add custom fields
      $customFieldHelper = new CustomFieldHelper(
        $this->container,
        $this->container->get('custom_field_set.repository')
      );
      $customFieldHelper->removeCustomField($context);

      $this->setPaymentMethodIsActive(FALSE, $context->getContext());
    }

    /**
     * activate
     *
     * @param \Shopware\Core\Framework\Plugin\Context\ActivateContext $context
     */
    public function activate(ActivateContext $context): void
    {
        parent::activate($context);
        $this->setPaymentMethodIsActive(TRUE, $context->getContext());
    }

    /**
     * deactivate
     *
     * @param \Shopware\Core\Framework\Plugin\Context\DeactivateContext $context
     */
    public function deactivate(DeactivateContext $context): void
    {
        parent::deactivate($context);
        $this->setPaymentMethodIsActive(FALSE, $context->getContext());
    }

    /**
     * getMethods
     *
     * @return string[]
     */
    private function getMethods()
    {
        $pms = array(
          CreditcardWeb::class  => array(
                'name' => 'Kreditkarte Micropayment',
                'desc' => 'powered by Micropayment',
            ),
          SepaWeb::class        => array(
                'name' => 'Lastschrift Micropayment',
                'desc' => 'powered by Micropayment',
            ),
          SofortWeb::class      => array(
                'name' => 'Sofort Micropayment',
                'desc' => 'powered by Micropayment',
            ),
          PaypalWeb::class      => array(
                'name' => 'PayPal Micropayment',
                'desc' => 'powered by Micropayment',
            ),
          PrepayWeb::class      => array(
                'name' => 'Vorkasse Micropayment',
                'desc' => 'powered by Micropayment',
            ),
          PaysafecardWeb::class => array(
                'name' => 'paysafecard Micropayment',
                'desc' => 'powered by Micropayment',
            ),
          PaydirektWeb::class   => array(
                'name' => 'paydirekt Micropayment',
                'desc' => 'powered by Micropayment',
            ),
          // deactivated due to POST Data problem on jump to thank you.
          // this needs to be fixed in MCP System or with workaround in plugin
          //            Call2payWeb::class    => array(
          //                'name' => 'Call2Pay Micropayment',
          //                'desc' => 'powered by Micropayment',
          //            ),
          //            HandypayWeb::class    => array(
          //                'name' => 'HandyPay Micropayment',
          //                'desc' => 'powered by Micropayment',
          //            ),
          // API Paymethods
          CreditcardApi::class  => array(
                'name' => 'Kreditkarte',
                'desc' => 'powered by Micropayment',
            ),
          SepaApi::class => array(
                'name' => 'Lastschrift',
                'desc' => 'powered by Micropayment',
            ),
          PrepayApi::class      => array(
                'name' => 'Vorkasse',
                'desc' => 'powered by Micropayment',
            ),
        );
        return $pms;
    }

    /**
     * addPaymentMethod
     *
     * @param \Shopware\Core\Framework\Context $context
     */
    private function addPaymentMethod(Context $context): void
    {
        $pms = $this->getMethods();
        $i = 0;
        foreach ($pms as $pm => $info) {
            $i++;
            $paymentMethodExists = $this->getPaymentMethodId($pm);

            /** @var PluginIdProvider $pluginIdProvider */
            $pluginIdProvider = $this->container->get(PluginIdProvider::class);
            $pluginId         = $pluginIdProvider->getPluginIdByBaseClass(\get_class($this), $context);



            /** @var EntityRepository $paymentRepository */
            $paymentRepository = $this->container->get('payment_method.repository');
            if ($paymentMethodExists) {
                $paymentMethodId = $this->getPaymentMethodId($pm);
                $paymentData = [
                    'id'                => $paymentMethodId,
                    'handlerIdentifier' => $pm,
                    'name'              => $info['name'],
                    'description'       => $info['desc'],
                    'pluginId'          => $pluginId,
                    'position'          => $i,
                    //'active'            => true,
                ];
                //continue; // skip if exist
                $paymentRepository->update([$paymentData], $context);
            } else {
                $paymentData = [
                    'handlerIdentifier' => $pm,
                    'name'              => $info['name'],
                    'description'       => $info['desc'],
                    'pluginId'          => $pluginId,
                    'position'          => $i,
                    //'active'            => true,
                ];
                $paymentRepository->create([$paymentData], $context);
            }
        }
    }

    /**
     * setPaymentMethodIsActive
     *
     * @param bool                             $active
     * @param \Shopware\Core\Framework\Context $context
     */
    private function setPaymentMethodIsActive(bool $active, Context $context): void
    {
        /** @var EntityRepository $paymentRepository */
        $paymentRepository = $this->container->get('payment_method.repository');

        if ($active) {
            /** @var \Micropayment\Helper\MediaHelper $mediaHelper */
            $mediaHelper = $this->container->get('Micropayment\Helper\MediaHelper');
        }

        $pms = $this->getMethods();
        foreach ($pms as $pm => $info) {
            $paymentMethodId = $this->getPaymentMethodId($pm);

            // Payment does not even exist, so nothing to (de-)activate here
            if (!$paymentMethodId) continue;

            $paymentMethod = [
                'id'      => $paymentMethodId,
                'active'  => $active,
            ];
            if ($active) {
                // Add payment methods
                $mediaId = $mediaHelper->getMediaId($pm, $context);
                $paymentMethod['mediaId'] = $mediaId;
            }

            $paymentRepository->update([$paymentMethod], $context);
        }
    }

    /**
     * getPaymentMethodId
     *
     * @param $pm
     *
     * @return string|null
     */
    private function getPaymentMethodId($pm): ?string
    {
        /** @var EntityRepository $paymentRepository */
        $paymentRepository = $this->container->get('payment_method.repository');

        // Fetch ID for update
        $paymentCriteria = (new Criteria())->addFilter(new EqualsFilter('handlerIdentifier', $pm));
        $paymentIds      = $paymentRepository->searchIds($paymentCriteria, Context::createDefaultContext());

        if ($paymentIds->getTotal() === 0) {
            return NULL;
        }

        return $paymentIds->getIds()[0];
    }
}
