<?php declare(strict_types=1);

namespace Micropayment\Service;

use Micropayment\Helper\McPayment;
use Shopware\Core\Checkout\Cart\Cart;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Checkout\Payment\Exception\CustomerCanceledAsyncPaymentException;
use Shopware\Core\Checkout\Payment\Exception\ValidatePreparedPaymentException;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\Struct\Struct;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\Locale\LocaleEntity;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\System\Language\LanguageCollection;
use Shopware\Core\Framework\Context;

class ApiPay implements AsynchronousPaymentHandlerInterface
{
  /**
   * @var \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler
   */
  private $transactionStateHandler;

  /** @var \Micropayment\Service\Config */
  protected $config;

  /** @var \Micropayment\Helper\McPayment */
  protected $mcp;

  /** @var \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository */
  protected $localeRepo;

  protected $paymethod;

  /**
   * WebPay constructor.
   *
   * @param \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler $transactionStateHandler
   * @param \Micropayment\Service\Config                                                          $config
   * @param \Micropayment\Helper\McPayment                                                        $mcp
   * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $localeRepo
   */
  public function __construct(
    OrderTransactionStateHandler $transactionStateHandler,
    Config                       $config,
    McPayment                    $mcp,
    EntityRepository    $localeRepo
  )
  {
    $this->transactionStateHandler = $transactionStateHandler;
    $this->config                  = $config;
    $this->mcp                     = $mcp;
    $this->localeRepo              = $localeRepo;
  }

  /**
   * pay
   *
   * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
   * @param \Shopware\Core\Framework\Validation\DataBag\RequestDataBag         $dataBag
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function pay(
    AsyncPaymentTransactionStruct $transaction,
    RequestDataBag                $dataBag,
    SalesChannelContext           $salesChannelContext
  ): RedirectResponse
  {
    if ($this->mcp->debug) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, 'REQUEST: '.print_r($_REQUEST, true)); // log error
    return new RedirectResponse('/');
  }

  /**
   * finalize
   *
   * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
   * @param \Symfony\Component\HttpFoundation\Request                          $request
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
   */
  public function finalize(
    AsyncPaymentTransactionStruct $transaction,
    Request                       $request,
    SalesChannelContext           $salesChannelContext
  ): void
  {
  }

  /**
   * getLocaleCodeFromContext
   *
   * @param String                                                 $languageId
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext $salesChannelContext
   *
   * @return string iso2
   */
  public function getLocaleCodeFromContext(string $languageId, SalesChannelContext $salesChannelContext): string
  {
    $criteria = new Criteria([$languageId]);
    $criteria->addAssociation('locale');

    $context          = $salesChannelContext->getContext();
    $localeRepository = $this->localeRepo;
    /** @var LanguageCollection $languageCollection */
    $languageCollection = $localeRepository->search($criteria, $context)->getEntities();

    $language = $languageCollection->get($languageId);
    if ($language === NULL) {
      return 'en';
    }

    $locale = $language->getLocale();
    if (!$locale) {
      return 'en';
    }

    return substr($locale->getCode(), 0, 2);
  }

  /**
   * getURL
   *
   * @param $context
   * @param $testMode
   * @param $status
   *
   * @return string
   */
  public function getURL($context, $testMode = FALSE, $status = 'notify')
  {
    $baseURL = $context->getSalesChannel()->getDomains()->first()->getUrl();
    if (!empty($testMode) && $this->mcp->isLocal()){
      $baseURL = 'http://web-dezign.de/resp/sendreq.php?p=';
    }
    switch ($status){
      case 'error':
        $tailURL = '/account/order/edit/';
        break;
      case 'notify':
      default:
        $tailURL = '/micropayment/notify/';
        break;
    }

    return $baseURL.$tailURL;
  }

}
