<?php declare(strict_types=1);

namespace Micropayment\Service;

use Micropayment\Helper\McPayment;
use Micropayment\Helper\PaymentStatusHelper;
use Micropayment\Helper\DataStruct;
//use Micropayment\Service\Method\CreditcardApi;
use Shopware\Storefront\Page\Account\Order\AccountEditOrderPageLoadedEvent;
use Shopware\Storefront\Page\Account\PaymentMethod\AccountPaymentMethodPageLoadedEvent;
use Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent;
//use Shopware\Storefront\Pagelet\Footer\FooterPageletLoadedEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
//use Shopware\Core\Framework\Struct\Struct;
use Symfony\Contracts\Translation\TranslatorInterface;

class Checkout implements EventSubscriberInterface
{
  /** @var \Micropayment\Helper\McPayment */
  protected $McPay;

  /** @var \Micropayment\Service\Config */
  protected $config;

  /** @var \Micropayment\Helper\PaymentStatusHelper */
  protected $psh;

  /** @var \Symfony\Contracts\Translation\TranslatorInterface */
  private TranslatorInterface $translator;


  public function __construct(
    McPayment           $McPay,
    Config              $config,
    PaymentStatusHelper $psh,
    TranslatorInterface $translator
  )
  {
    $this->McPay      = $McPay;
    $this->config     = $config;
    $this->psh        = $psh;
    $this->translator = $translator;

    //dd($config);
  }

  public static function getSubscribedEvents(): array
  {
    return [
      AccountEditOrderPageLoadedEvent::class => 'addPaymentForm',
      //      AccountPaymentMethodPageLoadedEvent::class => 'addPaymentForm',
      //      FooterPageletLoadedEvent::class => 'addPaymentForm',
      CheckoutConfirmPageLoadedEvent::class  => 'addPaymentForm',
    ];
  }

  /**
   * addPaymentForm
   *
   * @param Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent $event
   */
  public function addPaymentForm($event): void
  {
    //dd($event);
    $settings = $this->config->getSettings($event->getSalesChannelContext()->getSalesChannel()->getId());
    /**
     * Shopware\Core\System\SalesChannel $salesChannel
     */
    $salesChannel = $event->getSalesChannelContext();
    $pm           = $salesChannel->getPaymentMethod();
    $pmFHI        = $pm->getFormattedHandlerIdentifier();
    $pmID         = '';
    $dataStruct   = new DataStruct();
    $customer     = $salesChannel->getCustomer();
    $settings['guest'] = (bool)($customer->getGuest() == 1);
    //dd($customer);
    $settings['UID'] = $customer->customerNumber;

    $pmID = $pm->getID();
    $dataStruct->setPM($pmID);

    //echo '<pre>'.print_r($settings,true).'</pre>';
    $url                        = $event->getSalesChannelContext()->getSalesChannel()->getDomains()->first()->getUrl();
    $settings['plugin_urlshop'] = $url;
    $settings['McPayJSURL']     = $settings['plugin_urlshop'].'/micropayment/notify/?function=getScript&sn=';
    $settings['McPayCSSURL']    = $settings['plugin_urlshop'].'/micropayment/notify/?function=getStyle&sn=';
    $this->McPay->init($settings);
    //dd($this->McPay);
    //echo '<pre>'.print_r($this->McPay,true).'</pre>';

    // creditcard API
    if ($pmFHI == 'handler_micropayment_creditcardapi') {
      $form = $this->getCCardForm($settings);
      $dataStruct->setText($form);
      $extName = 'mcpay_ccard_form';
      // sepa API
    } else if ($pmFHI == 'handler_micropayment_sepaapi') {
      $form = $this->getSepaForm($settings);
      $dataStruct->setText($form);
      $extName = 'mcpay_sepa_form';
      // prepay API
    } else if ($pmFHI == 'handler_micropayment_prepayapi') {
      $form = $this->getPrepayForm($settings);
      $dataStruct->setText($form);
      $extName = 'mcpay_prepay_form';
    } else {
      return;
    }

    if ($event instanceof CheckoutConfirmPageLoadedEvent) {
      $confirmPage = $event->getPage();
      $confirmPage->addExtension($extName, $dataStruct);
      return;
    }

    if ($event instanceof AccountPaymentMethodPageLoadedEvent || $event instanceof AccountEditOrderPageLoadedEvent) {
      $event->getPage()->addExtension($extName, $dataStruct);
      return;
    }

    $event->getPagelet()->addExtension($extName, $dataStruct);
  }

  /**
   * getCCardForm
   *
   * @param $config
   *
   * @return string
   */
  private function getCCardForm($config)
  {
    $dir      = __DIR__.'/../../vendor/';
    $filename = $dir.'mcpay/view/mcpay_form_ccard.tpl';
    $options  = array(
      'showRemember'     => TRUE,  // show remember checkbox
      'showRemove'       => FALSE, // show remove checkbox
      'cardStyle'        => TRUE,  // display in card style
      'futureYears'      => 10,
      'preSeletActDate'  => FALSE,
      'showMonthNames'   => FALSE,
      'specialBrand'     => FALSE,
      'specialHolder'    => FALSE,
      'preselectNewCard' => FALSE,
      'prefillPost'      => $_POST,
      'hideImg'          => TRUE,  // hide logo
      'imgStyle'         => '',     // logo style
    );
    if (!empty($_POST['mcpay_card_reuse']) && $_POST['mcpay_card_reuse'] == 'new') {
      $options['preselectNewCard'] = TRUE;
    }
    if (!empty($config['guest'])){
      $options['showRemember'] = FALSE;
    }
    if (!empty($config['UID'])){
      $options['UID'] = $config['UID'];
    }
//    $accountMode = $this->container->get('session')->offsetGet('sOneTimeAccount');
//    if ($accountMode == 1){
//      $options['showRemember'] = FALSE;
//    }
    $form     = '';
    $errorMsg = '';
    if (empty($config['payformid'])) {
      $config['payformid'] = 'changePaymentForm';
    }

    // mark error fields
//    if (!empty(Shopware()->Session()->mcpay_options)) {
//      $options['classes']                  = Shopware()->Session()->mcpay_options;
//      $errorMsg                            = $this->getSnippet('McPayCheckInputs', 'Please check your inputs!');
//      Shopware()->Session()->mcpay_options = array(); // clean errors
//    }
//    if (!empty(Shopware()->Session()->mcpay_fields)) {
//      //echo '<pre>'.print_r(Shopware()->Session()->mcpay_sepa_fields,1).'</pre>';
//      $options['checkRemember'] = Shopware()->Session()->mcpay_fields['mcpay_card_remember'];
//      $options['prefillPost']   = Shopware()->Session()->mcpay_fields;
//    }

    if (!empty($errorMsg)) {
      $form .= '<br><div class="'.$this->McPay->prefixCCard.'errorBox">'.$errorMsg.'</div>';
    }

    // translations to the default texts in template file. Search and replace is done.
    $translation = array(
      'Name on Card'              => $this->getSnippet('micropayment.checkout.ccard.holder'),
      'Remember Data'             => $this->getSnippet('micropayment.checkout.ccard.RememberCardData'),
      'Use new Card'              => $this->getSnippet('micropayment.checkout.ccard.UseNewCCard'),
      'Use old Card'              => $this->getSnippet('micropayment.checkout.ccard.UseOldCCard'),
      'Please check your inputs!' => $this->getSnippet('micropayment.checkout.ccard.CheckInputs'),
      'Vorname Nachname'          => $this->getSnippet('micropayment.checkout.ccard.Name'),
      'Credit Card Number'        => $this->getSnippet('micropayment.checkout.ccard.pan'),
      'Card Number'               => $this->getSnippet('micropayment.checkout.ccard.CCardNumber'),
      'Expiration Date'           => $this->getSnippet('micropayment.checkout.ccard.ExpDate'),
      'ExpDate'                   => $this->getSnippet('micropayment.checkout.ccard.ExpDate'),
      'Card CVC'                  => $this->getSnippet('micropayment.checkout.ccard.CVC'),
      'Month'                     => $this->getSnippet('micropayment.checkout.ccard.Month'),
      'Year'                      => $this->getSnippet('micropayment.checkout.ccard.Year'),
      'NAME'                      => $this->getSnippet('micropayment.checkout.ccard.holder'),
      'PAN'                       => $this->getSnippet('micropayment.checkout.ccard.PAN'),
      'EXPIRE'                    => $this->getSnippet('micropayment.checkout.ccard.EXPIRE'),
    );

    $prefillData = array();
//    if (!empty($req['pmid'])) {
//      $hasSavedData = $this->loadFields($req['pmid']);
//      if (!empty($hasSavedData)) {
//        $prefillData[1] = array(
//          'holder'   => $hasSavedData['mcpay_card_holder'],
//          'month'    => $hasSavedData['mcpay_card_month'],
//          'year'     => $hasSavedData['mcpay_card_year'],
//          'pan'      => $hasSavedData['mcpay_card_pan'],
//          'cvc'      => $hasSavedData['mcpay_card_cvc'],
//          'token'    => $hasSavedData['mcpay_card_token'],
//          'remember' => $hasSavedData['mcpay_card_remember'],
//        );
//      }
//    }

//    $scriptURL = $this->Front()->Router()->assemble(array('forceSecure' => 1, 'action' => 'getScript', 'controller' => 'MicropaymentCcard')).'/sn/';
//    $styleURL  = $this->Front()->Router()->assemble(array('forceSecure' => 1, 'action' => 'getStyle', 'controller' => 'MicropaymentCcard')).'/sn/';#
    // http://php74.docker:8080/sw6/public/micropayment/notify/?function=getScript&sn=mcpay_formstyle_ccard_check
    $scriptURL = $config['McPayJSURL'];
    $styleURL  = $config['McPayCSSURL'];
    $form      .= $this->McPay->getCCardTPL($filename, $options, $prefillData, $translation);
    $form      .= '<script>
        function mcpay_ccard_execute()
        {
            var f = getElem("'.$config['payformid'].'");
            var s = document.createElement("script");
            s.setAttribute( "src", "'.$scriptURL.'mcpay_formstyle_ccard_check" );
            s.setAttribute( "type", "text/javascript");
            f.appendChild(s);
            return true;
        }
        getElem = function (name) {
            return document.getElementById(name);
        }
    </script>';
    //$form        .= '<script src="js/mcpay_formstyle_ccard_check.js"></script>';
    //$form        .= '<script src="js/mcpay_formstyle_ccard_iframe.js"></script>';
    //$form .= '<input type="submit" name="submit" value="PAY">';
    $form .= '<script>';
    if ($this->McPay->debug) {
      $form .= 'console.log("MCPAY ModelCCard START"); ';
    }

    $inlineCode = $this->McPay->getStartJS(TRUE, FALSE, $this->McPay->debug, $config['payformid']);
    $form       .= 'mcpay_ccard_execute(); ';
    $form       .= $inlineCode.' ';

    if ($this->McPay->debug) {
      $form .= 'console.log("MCPAY ModelCCard END"); ';
    }
    $form .= '</script>';
    $form .= '<link href="'.$styleURL.'mcpay_formstyle_ccard" rel="stylesheet" />';
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$form', print_r($form,1));

    return $form;

//    $this->View()->assign('form', $form);
//    $this->View()->assign('payformid', $config['payformid']);
  }

  /**
   * getSepaForm
   *
   * @param $config
   *
   * @return string
   */
  private function getSepaForm($config)
  {
    $dir      = __DIR__.'/../../vendor/';
    $filename = $dir.'mcpay/view/mcpay_form_sepa.tpl';
    $options  = array(
      'showRemember' => TRUE,  // show remember checkbox
      'showRemove'   => FALSE, // show remove checkbox
      'cardStyle'    => TRUE,  // display in card style
      'showCardOnly' => FALSE, // disable radio button for cards
      'hideImg'      => TRUE,  // hide logo
      'imgStyle'     => '',     // logo style
      'classes'      => array(
        'holder' => '',
        'iban'   => '',
        'bic'    => '',
      ),
    );
    if (!empty($_POST['mcpay_card_reuse']) && $_POST['mcpay_card_reuse'] == 'new') {
      $options['preselectNewCard'] = TRUE;
    }
    if (!empty($config['guest'])){
      $options['showRemember'] = FALSE;
    }
    if (!empty($config['UID'])){
      $options['UID'] = $config['UID'];
    }

    $form     = '';
    $errorMsg = '';
    if (empty($config['payformid'])) {
      $config['payformid'] = 'changePaymentForm';
    }

    if (!empty($errorMsg)) {
      $form .= '<br><div class="'.$this->McPay->prefixCCard.'errorBox">'.$errorMsg.'</div>';
    }

    // translations to the default texts in template file. Search and replace is done.
    $translation = array(
      'Name on Card'              => $this->getSnippet('micropayment.checkout.sepa.holder'),
      'Remember Data'             => $this->getSnippet('micropayment.checkout.sepa.RememberCardData'),
      'Use new Card'              => $this->getSnippet('micropayment.checkout.sepa.UseNewCard'),
      'Use old Card'              => $this->getSnippet('micropayment.checkout.sepa.UseOldCard'),
      'Holder'                    => $this->getSnippet('micropayment.checkout.sepa.holder'),
      'IBAN'                      => $this->getSnippet('micropayment.checkout.sepa.IBAN'),
      'BIC'                       => $this->getSnippet('micropayment.checkout.sepa.BIC'),
      'Please check your inputs!' => $this->getSnippet('micropayment.checkout.sepa.CheckInputs'),
      'Vorname Nachname'          => $this->getSnippet('micropayment.checkout.sepa.Name'),
      'NAME'                      => $this->getSnippet('micropayment.checkout.sepa.holder'),
    );

    $prefillData = array();

    $scriptURL = $config['McPayJSURL'];
    $styleURL  = $config['McPayCSSURL'];
    $form      .= $this->McPay->getSepaTPL($filename, $options, $prefillData, $translation);
    $form      .= '<script src="'.$scriptURL.'mcpay_formstyle_sepa"></script>';
    $form      .= '<link href="'.$styleURL.'mcpay_formstyle_sepa" rel="stylesheet" />';

    return $form;
  }

  /**
   * getPrepayForm
   *
   * @param $config
   *
   * @return string
   */
  private function getPrepayForm($config)
  {
    $form = $this->getSnippet('micropayment.checkout.prepay.text');
    return $form;
  }

  /**
   * getSnippet
   *
   * @param string $snippet
   * @param array  $parameters
   *
   * @return string
   */
  protected function getSnippet(string $snippet, array $parameters = []): string
  {
    //dd($this->translator);
    return $this->translator->trans($snippet, $parameters);
  }

}