<?php

namespace Micropayment\Service\Method;

use Micropayment\Helper\McPayment;
use Micropayment\Service\Config;
use Micropayment\Service\ApiPay;
use Micropayment\Helper\PaymentStatusHelper;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;

class SepaApi extends ApiPay
{
  public const PAYMENT_METHOD_NAME = 'SEPA_API';

  /** @var \Micropayment\Helper\PaymentStatusHelper */
  protected $psh;

  /**
   * @param \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler $transactionStateHandler
   * @param \Micropayment\Service\Config                                                          $config
   * @param \Micropayment\Helper\McPayment                                                        $mcp
   * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $localeRepo
   * @param \Micropayment\Helper\PaymentStatusHelper                                              $psh
   */
  public function __construct(
    OrderTransactionStateHandler $transactionStateHandler,
    Config                       $config,
    McPayment                    $mcp,
    EntityRepository    $localeRepo,
    PaymentStatusHelper          $psh
  )
  {
    $this->paymethod = self::PAYMENT_METHOD_NAME;
    $this->psh       = $psh;
    parent::__construct($transactionStateHandler, $config, $mcp, $localeRepo);
  }

  /**
   * pay
   *
   * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
   * @param \Shopware\Core\Framework\Validation\DataBag\RequestDataBag         $dataBag
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function pay(
    AsyncPaymentTransactionStruct $transaction,
    RequestDataBag                $dataBag,
    SalesChannelContext           $salesChannelContext
  ): RedirectResponse
  {
    try {
      //dd($_REQUEST);
      if (!empty($_REQUEST['McPay_SepaOLDreuse']) && $_REQUEST['McPay_SepaOLDreuse'] != 'new'){
        if (empty($_REQUEST['McPay_SepaHOLDERreuse'])) {
          throw new AsyncPaymentProcessException(
            $transaction->getOrderTransaction()->getId(),
            'Holder cant be empty!'.PHP_EOL
          );
        }
        if (empty($_REQUEST['McPay_SepaIBANreuse'])) {
          throw new AsyncPaymentProcessException(
            $transaction->getOrderTransaction()->getId(),
            'IBAN cant be empty!'.PHP_EOL
          );
        }
        if (substr(strtoupper($_REQUEST['McPay_SepaIBANreuse']), 0, 2) != 'DE' && empty($_REQUEST['McPay_SepaBICreuse'])) {
          throw new AsyncPaymentProcessException(
            $transaction->getOrderTransaction()->getId(),
            'BIC cant be empty!'.PHP_EOL
          );
        }
      } else {
        if (empty($_REQUEST['McPay_SepaHOLDER'])) {
          throw new AsyncPaymentProcessException(
            $transaction->getOrderTransaction()->getId(),
            'Holder cant be empty!'.PHP_EOL
          );
        }
        if (empty($_REQUEST['McPay_SepaIBAN'])) {
          throw new AsyncPaymentProcessException(
            $transaction->getOrderTransaction()->getId(),
            'IBAN cant be empty!'.PHP_EOL
          );
        }
        if (substr(strtoupper($_REQUEST['McPay_SepaIBAN']), 0, 2) != 'DE' && empty($_REQUEST['McPay_SepaBIC'])) {
          throw new AsyncPaymentProcessException(
            $transaction->getOrderTransaction()->getId(),
            'BIC cant be empty!'.PHP_EOL
          );
        }
      }

      $settings = $this->config->getSettings($salesChannelContext->getSalesChannel()->getId());
      $settings['paymethod'] = 'sepa';
      $config = $this->mcp->init($settings);
      if ($this->mcp->debug) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, print_r($_REQUEST, true)); // log error

      $order    = $transaction->getOrder();
      $userData = $this->mcp->getUserData($order, $settings, $salesChannelContext);

      $langISO             = $this->getLocaleCodeFromContext($order->getLanguageId(), $salesChannelContext);
      $userData['lang']    = $langISO;
      $orderID             = $order->getId();
      $trxID               = $transaction->getOrderTransaction()->getId();
      $userData['trxid']   = $trxID;
      $userData['oid']     = $order->getOrderNumber(); // $order->getId();
      $userData['respurl'] = urlencode($transaction->getReturnUrl()); // final thank you url

      $notifyURL = $this->getURL($salesChannelContext, $config['testMode'], 'notify');
      $errorURL = $this->getURL($salesChannelContext, FALSE, 'error').$orderID.'?error-code=CHECKOUT__ASYNC_PAYMENT_PROCESS_INTERRUPTED';

      $query = '&orderid='.$userData['oid'].'&trxid='.$trxID
        .'&returnURL='.urlencode($userData['respurl'])
        .'&errorURL='.urlencode(urlencode($errorURL));
      $config['returnURL'] = $notifyURL.'?function=handle3D'.$query;
      $config['autogenSession'] = TRUE;
      $config['isAbo'] = FALSE;

      if (empty($_REQUEST['McPay_SepaREMEMBER'])){
        $_REQUEST['McPay_SepaREMEMBER'] = 0;
      }
      $payData = array(
        'holder'   => $_REQUEST['McPay_SepaHOLDER'],
        'iban'     => $_REQUEST['McPay_SepaIBAN'],
        'bic'      => $_REQUEST['McPay_SepaBIC'],
        'remember' => $_REQUEST['McPay_SepaREMEMBER'],
        'bankName' => '',
      );
      $payData['reuse'] = FALSE;
      if (!empty($_REQUEST['McPay_SepaOLDreuse']) && $_REQUEST['McPay_SepaOLDreuse'] != 'new'){
        $payData = array(
          'holder'   => $_REQUEST['McPay_SepaHOLDERreuse'],
          'iban'     => $_REQUEST['McPay_SepaIBANreuse'],
          'bic'      => $_REQUEST['McPay_SepaBICreuse'],
          'remember' => 0,
          'bankName' => '',
          'reuse'    => true,
        );
      }
      if (!empty($userData['guest'])){
        $payData['reuse'] = false; // guest uses always new cards
      }
      $res = $this->mcp->bookSepaShort($userData, $config, $payData);
      //dd($res);

      $this->setStatus($salesChannelContext, $trxID, $userData['order_id'], $res);

      // jump to thank you page
      $redirectUrl = urldecode($userData['respurl']);

    } catch (\Exception $e) {
      if ($this->mcp->debug) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage().' - '.$e->getCode()); // log error
      if ($this->mcp->debug && !empty($res)) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, true)); // log error

      throw new AsyncPaymentProcessException(
        $transaction->getOrderTransaction()->getId(),
        'An error occurred during the communication with external payment gateway'.PHP_EOL.$e->getMessage()
      );
    }

    // Redirect to external gateway
    return new RedirectResponse($redirectUrl);
  }

  /**
   * setStatus
   *
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext $SalesChannelContext
   */
  private function setStatus($SalesChannelContext, $trxid, $orderId, $res)
  {
    $comment = $res->mandateTextDE;
    if (strtolower($res->custObj->lang) != 'de'){
      $comment = $res->mandateTextEN;
    }

    $context = $SalesChannelContext->getContext();
    $state   = 'paid';

    $this->psh->setState($trxid, $state, $context);

    $fields  = array(
      'mcpay_authid'   => $res->resCreate->payToken,
      'mcpay_trxid'    => $trxid,
      'mcpay_comments' => $comment,
    );
    $this->psh->setCustomFields($orderId, $fields, $context);
  }
}
