<?php

namespace Micropayment\Service\Method;

use Micropayment\Helper\McPayment;
use Micropayment\Service\Config;
use Micropayment\Service\WebPay;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\SyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\SynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Checkout\Payment\Exception\SyncPaymentProcessException;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\Language\LanguageCollection;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;

class SepaApiSync implements SynchronousPaymentHandlerInterface
{
    public const PAYMENT_METHOD_NAME = 'SEPA_API';

    /** @var \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler  */
    var $tsh;
    /** @var McPayment */
    var $mcp;
    /** @var Config */
    var $config;
    /** @var \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository */
    var $localeRepo;

    /**
     * SepaWeb constructor.
     *
     * @param \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler $transactionStateHandler
     * @param \Micropayment\Service\Config                                                          $config
     * @param \Micropayment\Helper\McPayment                                                        $mcp
     * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $localeRepo
     */
    public function __construct(
        OrderTransactionStateHandler $transactionStateHandler,
        Config $config,
        McPayment $mcp,
        EntityRepository $localeRepo
    )
    {
        $this->tsh = $transactionStateHandler;
        $this->paymethod = self::PAYMENT_METHOD_NAME;
        $this->mcp = $mcp;
        $this->config = $config;
        $this->localeRepo = $localeRepo;
        //parent::__construct($transactionStateHandler, $config, $mcp, $localeRepo);
    }

    /**
     * pay
     *
     * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
     * @param \Shopware\Core\Framework\Validation\DataBag\RequestDataBag         $dataBag
     * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function pay(
        SyncPaymentTransactionStruct $transaction,
        RequestDataBag $dataBag,
        SalesChannelContext $salesChannelContext
    ): void {
        //echo '<pre>'.print_r($dataBag->all(),true).'</pre>';  exit();
        //echo '<pre>'.print_r($_POST,true).'</pre>'; exit();
        try {
            $req = $dataBag->all();
            //echo '<pre>'.print_r($req,true).'</pre>'; exit();
            if (empty($req['mcpay_sepa_holder'])){
                //$this->tsh->process($transaction->getOrderTransaction()->getId(), $salesChannelContext->getContext());
                throw new SyncPaymentProcessException(
                    $transaction->getOrderTransaction()->getId(),
                    'Holder cant be empty!'
                );
            }

            $settings = $this->config->getSettings($salesChannelContext->getSalesChannel()->getId());
            $settings['paymethod'] = 'sepa';
            $config = $this->mcp->init($settings);
            //echo '<pre>'.print_r($settings,true).'</pre>'; exit();
            //echo '<pre>'.print_r($this->mcp,true).'</pre>'; exit();

            $order = $transaction->getOrder();
            $userData = $this->mcp->getUserData($order, $settings, $salesChannelContext);

            $langISO = $this->getLocaleCodeFromContext($order->getLanguageId(), $salesChannelContext);
            $userData['lang'] = $langISO;
            $trxID = $transaction->getOrderTransaction()->getId();
            $userData['trxid'] = $trxID;
            $userData['oid'] = $order->getId();
            //$userData['respurl'] = urlencode($transaction->getReturnUrl());

//            $p = explode('_', $this->paymethod);
//            $activePayment = strtolower($p[0]);
//            $redirectUrl = $this->mcp->getURL($userData, $activePayment);

            if (empty($req['mcpay_sepa_remember'])){
                $req['mcpay_sepa_remember'] = 0;
            }
            $payData = array(
                'holder'   => $req['mcpay_sepa_holder'],
                'iban'     => $req['mcpay_sepa_iban'],
                'bic'      => $req['mcpay_sepa_bic'],
                'remember' => $req['mcpay_sepa_remember'],
                'bankName' => '',
            );
            $payData['reuse'] = FALSE;
            $res = $this->mcp->bookSepaShort($userData, $config, $payData);
            echo '<pre>'.print_r($res,true).'</pre>';
            exit();

        } catch (\Exception $e) {
            $this->mcp->log->debug($e->getMessage().' in '.$e->getFile().' Line '.$e->getLine());
            throw new AsyncPaymentProcessException(
                $transaction->getOrderTransaction()->getId(),
                'An error occurred during the communication with external payment gateway' . PHP_EOL . $e->getMessage()
            );
        }

        // Redirect to external gateway
        //return new RedirectResponse($redirectUrl);
    }

    /**
     * getLocaleCodeFromContext
     *
     * @param String                                                 $languageId
     * @param \Shopware\Core\System\SalesChannel\SalesChannelContext $salesChannelContext
     *
     * @return string iso2
     */
    public function getLocaleCodeFromContext(String $languageId, SalesChannelContext $salesChannelContext): string
    {
        $criteria = new Criteria([$languageId]);
        $criteria->addAssociation('locale');

        $context = $salesChannelContext->getContext();
        $localeRepository = $this->localeRepo;
        /** @var LanguageCollection $languageCollection */
        $languageCollection = $localeRepository->search($criteria, $context)->getEntities();

        $language = $languageCollection->get($languageId);
        if ($language === null) {
            return 'en';
        }

        $locale = $language->getLocale();
        if (!$locale) {
            return 'en';
        }

        return substr($locale->getCode(), 0, 2);
    }
}
