<?php declare(strict_types=1);

namespace Micropayment\Storefront\Controller;

use Micropayment\Helper\McPayment;
use Micropayment\Service\Config;
use Micropayment\Helper\PaymentStatusHelper;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Controller\StorefrontController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

#[Route(defaults: ['_routeScope' => ['storefront']])]
#[Package('storefront')]
class NotifyController extends StorefrontController
{
  /** @var \Micropayment\Helper\McPayment */
  protected $McPay;

  /** @var \Micropayment\Service\Config */
  protected $config;

  /** @var \Micropayment\Helper\PaymentStatusHelper */
  protected $psh;


  public function __construct(
    McPayment           $McPay,
    Config              $config,
    PaymentStatusHelper $psh
  )
  {
    $this->McPay  = $McPay;
    $this->config = $config;
    $this->psh    = $psh;
  }

  #[Route(path:"/micropayment/notify/", name:"frontend.micropayment.notify", options:["seo"=>"false"], methods:["GET", "POST"], defaults:["csrf_protected"=>false])]
  /**
   * @ Route("/micropayment/notify/", defaults={"csrf_protected"=false}, name="frontend.micropayment.notify",
   *                                           options={"seo"="false"}, methods={"GET", "POST"})
   * notify
   *
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext $SalesChannelContext
   *
   * @return \Symfony\Component\HttpFoundation\Response|null
   */
  public function notify(
    SalesChannelContext $SalesChannelContext
  ): ?Response
  {
    //echo '<pre>'.print_r(func_get_args(),true).'</pre>';
    //echo __FUNCTION__.' called!';
    $settings = $this->config->getSettings($SalesChannelContext->getSalesChannel()->getId());
    //echo '<pre>'.print_r($settings,true).'</pre>';
    $MPconfig = $this->McPay->init($settings);
    //echo '<pre>'.print_r($this->mcp,true).'</pre>';

    $url                 = $SalesChannelContext->getSalesChannel()->getDomains()->first()->getUrl();
    $settings['shopURL'] = $url;

    $return = '';
    $func   = '';
    $crlf   = "\r\n";
    $cfg    = $settings;
    $post   = $_POST;
    if (empty($post)) {
      $post = $_GET;
    }
    $amount   = 0;
    $currency = 'EUR';
    //echo '<pre>POST:'.print_r($post,true).'</pre>'; exit();

    if (!empty($post['secretfield'])) {
      $secretfield = $cfg['McPaySECRETFIELD'];
      if ($secretfield != $post['secretfield']) {
        $return = 'status=error'.$crlf.'message=Communication is insecure.';
      }
    }

    // api notifications
    if (!empty($_POST['action'])) {
      $importantActions = array('transactionCreate', 'transactionRefund', 'transactionChargeback', 'transactionBackpay','sessionStatus');
      if (!in_array($_POST['action'], $importantActions)) {
        $return = 'status=ok'.$crlf.'message=Action '.$_POST['action'];
        if (!empty($_POST['type'])){
          $return.= '->'.$_POST['type'];
        }
        $return.= ' ignored.';
        return new Response($return);
      }
      if ($_POST['action'] == 'sessionStatus'){
        $post['function'] = $_POST['status'];
        $post['auth'] = 'isPaid';
      } else {
        $post['function'] = $_POST['type'];
        $post['auth'] = $_POST['transactionId'];
      }
      $post['trxid'] = 'seeOrder';
      $post['testmode'] = $_POST['testMode'];
      $post['amount'] = $_POST['paidAmount'];
      $post['orderid'] = $_POST['sessionId'];
    }
    
    // check mandatory params
    if (empty($post['function'])) $return = 'status=error'.$crlf.'message=Function missing.';
    else                            $func = $post['function'];

    //dd($func);
    if ($func == 'getScript') {
      return $this->getScriptAction($cfg);
    }
    if ($func == 'getStyle') {
      return $this->getStyleAction($cfg);
    }
    if ($func == 'getImage') {
      return $this->getImageAction($cfg);
    }

    $context = $SalesChannelContext->getContext();

    // for testing
    if ($func == 'handleTEST') {
      return $this->handleTESTAction($post, $context, $MPconfig);
    }
    if ($func == 'handle3D') {
      return $this->handle3DAction($post, $context, $MPconfig);
    }
    if ($func == 'handleNO3D') {
      return $this->handleNO3DAction($post, $context);
    }


    if ($func != 'custinfo') {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->post'.print_r($post, TRUE));

      //if (empty($post['intresp'])) $return = 'status=error'.$crlf.'message=Please use responseAction as notification url.';

      if (empty($post['orderid'])) $return = 'status=error'.$crlf.'message=OrderID missing.';
      else                          $orderId = $post['orderid'];

      if (empty($post['auth'])) $return = 'status=error'.$crlf.'message=Auth missing.';
      else                          $authId = $post['auth'];

      if (empty($post['amount'])) $return = 'status=error'.$crlf.'message=Amount missing.';
      else                          $amount = $post['amount'];
      /*
      if (empty($post['currency'])) $return = 'status=error'.$crlf.'message=Currency missing.';
      else                          $currency = $post['currency'];
      */
      if (!isset($post['testmode'])) $return = 'status=error'.$crlf.'message=Testmode missing.';
      else                          $testmode = $post['testmode'];

      if (empty($post['trxid'])) $return = 'status=error'.$crlf.'message=TRX ID missing.';
      else                          $trxid = $post['trxid'];

      $oid = NULL;
      if (!empty($post['oid'])) $oid = $post['oid'];

      $respurl = '';
      if (empty($post['intresp']) && in_array($func, array('billing', 'init'))) {
//        if (empty($post['respurl'])) $return = 'status=error'.$crlf.'message=respurl missing.';
//        else                          $respurl = $post['respurl'];

        if (!empty($post['respurl'])){
          $respurl = $post['respurl'];
        }
      }

      $currency = 'EUR';
      if (!empty($post['currency'])) $currency = $post['currency'];

      $theAmount = sprintf('%1.2f', $amount / 100).' '.$currency;

    } else {
      if (empty($post['ak'])) $return = 'status=error'.$crlf.'message=Missing KEY!';
      else                      $ak = $post['ak'];
    }

    if (!empty($return)) {
      //echo $return;
      return new Response($return);
    }
    // default error after pre-checks
    $return = 'status=error'.$crlf.'message=Unknown Error';

    switch ($func) {

      // status call
      case 'custinfo':
        //echo '<pre>'.print_r($cfg,1).'</pre>';
        $accessKey = $cfg['McPayACCESSKEY'];
        //echo $accessKey.' == '.$ak;
        if ($accessKey == $ak) {
          if (!empty($post['log'])) {
            //echo $this->McPay->readLogFile($this->McPay->log->error_log, TRUE);
            echo $this->McPay->readerLogFile($this->McPay->log->error_log);
            exit();
          } else {
            /*
            $activePayment = $plugin->getActivePaymethods();
            echo '<pre>Paymentmeans:'.print_r($activePayment, 1).'</pre>';
            */
            $context    = $SalesChannelContext->getContext();
            $pluginInfo = $this->psh->getPluginDetails('micropayment', $context);
            $infos      = 'Shopversion: '.$this->psh->getShopVersion().'<br>';
            $infos      .= 'Pluginversion: '.$pluginInfo->getVersion().'<br>';
            $infos      .= '<textarea>'.print_r($settings, TRUE).'</textarea>';

            echo '<pre>'.print_r($infos, TRUE).'</pre>';
            exit();
          }
        } else {
          $return = 'status=error'.$crlf.'message=Access denied!';
        }
        break;

      case 'BOOKING':
        try {
          $context = $SalesChannelContext->getContext();
          $state   = 'paid';
          $this->psh->setState($trxid, $state, $context);

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      case 'CREATE':
      case 'CHANGE':
        try {
          $context = $SalesChannelContext->getContext();
          $state   = 'reopen';
          $this->psh->setState($trxid, $state, $context);

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      // billing call
      case 'billing':
      case 'init':
        try {
          $context = $SalesChannelContext->getContext();
          $state   = 'paid';
          if (in_array($func, array('init'))) {
            //$state = 'remind';
            $state = 'processUnconfirmed';
          }
          $this->psh->setState($trxid, $state, $context);
          $returnURL = '/checkout/finish?orderId='.$oid;
          $url       = $returnURL;
          if (!empty($respurl)) {
            $url = urldecode($respurl);
          }

          $comment = date('d.m.Y H:i:s').': '.$func;
          $fields  = array(
            'mcpay_authid'   => $authId,
            'mcpay_trxid'    => $trxid,
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf // ok  | error
            .'url='.$url.$crlf
            .'target=_top'.$crlf
            .'forward=1'.$crlf
            .'message=OK'.$crlf;

          //echo '<pre>'.print_r($_GET, TRUE).'</pre>';
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      case 'payin': // payin prepayment
      case 'PAYIN':
      case 'PAID':
        try {
          $context     = $SalesChannelContext->getContext();
          if (!empty($oid)){
            $transaction = $this->psh->getOrderTransaction($oid, $context);
          } else {
            $transaction = $this->psh->getOrderTransactionByOrderNumber($orderId, $context);
          }
          if ($trxid == 'seeOrder'){
            $trxid = $transaction->getId();
          }
          $actState = $transaction->getStateMachineState()->getTechnicalName();
          //dd($actState);
          $orderAmount = $transaction->getAmount()->getTotalPrice() * 100;
          //echo '<pre>'.$amount.' < '.print_r($orderAmount,true).'</pre>';
          $state = 'payPartially'; // not allowed yet by state machine transition rules
          if ($amount >= $orderAmount) {
            $state = 'paid';
            //$this->psh->setState($trxid, $state, $context);
          }
          if ($actState != 'paid') { // only update state if not paid
            $this->psh->setState($trxid, $state, $context);
          }

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      case 'expire': // expire prepayment
        try {
          $context = $SalesChannelContext->getContext();
          $state   = 'cancel';
          $this->psh->setState($trxid, $state, $context);

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      case 'storno': // chargeback sepa
      case 'transactionChargeback': // chargeback ccard
        try {
          $context = $SalesChannelContext->getContext();
          $state   = 'cancel';
          $this->psh->setState($trxid, $state, $context);

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      case 'backpay': // chargeback reversal
      case 'REFUNDREVERSAL'; // refund reversal
      case 'transactionBackpay': // backpay ccard
        try {
          $context = $SalesChannelContext->getContext();
          $state   = 'reopen';
          $this->psh->setState($trxid, $state, $context);

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      case 'refund': // refund sepa
      case 'REFUND': // refund prepayment
      case 'transactionRefund': // refund ccard
        try {
          $context     = $SalesChannelContext->getContext();
          $transaction = $this->psh->getOrderTransaction($oid, $context);
          $orderAmount = $transaction->getAmount()->getTotalPrice() * 100;
          //echo '<pre>'.print_r($orderAmount,true).'</pre>';
          $state = 'refund';
          if ($amount < $orderAmount) {
            $state = 'refundPartially';
          }
          $this->psh->setState($trxid, $state, $context);

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;

      case 'quit': // reversal SEPA only
        try {
          $context = $SalesChannelContext->getContext();
          $state   = 'cancel';
          $this->psh->setState($trxid, $state, $context);

          $comment = date('d.m.Y H:i:s').': '.$state.' Amount: '.$theAmount;
          $fields  = array(
            'mcpay_comments' => $comment,
          );
          $this->psh->setCustomFields($orderId, $fields, $context);

          $return = 'status=ok'.$crlf.'message='.$state.' done.'.$crlf;
        } catch (Exception $e) {
          $return = 'status=fail'.$crlf.'message='.$e->getMessage().$crlf;
        }
        break;
    }

    $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->return: '.$return);

    return new Response($return);
  }

  /**
   * getStyleAction
   *
   * @param $config
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function getStyleAction($config)
  {
    $req = array();
    if (!empty($_GET)) {
      $req = $_GET;
    }
    if (empty($req['sn'])) {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      return new Response('param sn missing!', 403);
    }

    $filename = MCPAY_BASE_PATH.'css/'.$req['sn'].'.css';
    if (!file_exists($filename)) {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      return new Response('file not found!', 404);
    }
    $content = file_get_contents($filename);

    // icon paths
    if (strpos($filename, 'mcpay_formstyle_') !== FALSE) {
      //$baseURL = $this->Front()->Router()->assemble(array('forceSecure' => 1, 'action' => 'getImage', 'controller' => 'MicropaymentCcard')).'/sn/';
      $baseURL = $config['shopURL'].'/micropayment/notify/?function=getImage&sn=';
      $repl    = array(
        'url("../img/icon_' => 'url("'.$baseURL.'icon_',
        '.png'              => '',
        'user'              => 'use',
      );
      $content = strtr($content, $repl);
    }

    return new Response($content, 200, array('Content-Type' => 'text/css'));
  }

  /**
   * getScriptAction
   *
   * @param $config
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function getScriptAction($config)
  {
    $req = array();
    if (!empty($_GET)) {
      $req = $_GET;
    }
    if (empty($req['sn'])) {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      return new Response('param sn missing!', 403);
    }

    $filename = MCPAY_BASE_PATH.'js/'.$req['sn'].'.js';
    if (!file_exists($filename)) {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      return new Response('file not found!', 404);
    }
    if (empty($config['payformid'])) {
      $config['payformid'] = 'confirm--form';
    }

    $content = file_get_contents($filename);
    if (strpos($filename, 'check.js') !== FALSE) {
      $repl    = array(
        'payformid'             => $config['payformid'],
        'mcpay_card_token-form' => $config['payformid'],
      );
      $content = strtr($content, $repl);
    }

    return new Response($content, 200, array('Content-Type' => 'application/javascript'));
  }

  /**
   * getImageAction
   *
   * @param $config
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function getImageAction($config)
  {
    $req = array();
    if (!empty($_GET)) {
      $req = $_GET;
    }
    if (empty($req['sn'])) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      return new Response('param sn missing!', 403);
    }

    $filename = MCPAY_BASE_PATH.'img/'.$req['sn'].'.png';
    if (!file_exists($filename)) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      return new Response('file not found!', 404);
    }
    $content = file_get_contents($filename);
    return new Response($content, 200, array('Content-Type' => 'image/png'));
  }

  /**
   * handle3DAction
   *
   * @param $post
   * @param $context
   * @param $config
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function handle3DAction($post, $context, $config)
  {
    $crlf   = "\r\n";
    $orderId  = $post['orderid'];
    $trxid    = $post['trxid'];
    $returnURL = urldecode($post['returnURL']);
    $errorURL = urldecode($post['errorURL']);

    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->post'.print_r($_REQUEST, TRUE));

    if (!empty($_REQUEST['check3dId'])) {
      $check3dId = $_REQUEST['check3dId'];
    } else {
      return new Response('3D Secure failed!');
    }

    try{
      $res = $this->McPay->verify3D($check3dId, $config);
    } catch (\Exception $e) {
      $res = new \stdClass();
      $res->error = 1;
      $res->errorMSG = $e->getMessage();
    }
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->verify3D: '.$check3dId.print_r($res, TRUE));

    if (isset($res->error) && $res->error == 0) {
      $state = 'paid';
      $this->psh->setState($trxid, $state, $context);

      $comment = date('d.m.Y H:i:s').': handle3D';
      $fields  = array(
        'mcpay_authid'   => $check3dId,
        'mcpay_trxid'    => $trxid,
        'mcpay_comments' => $comment,
      );
      $this->psh->setCustomFields($orderId, $fields, $context);

      return new Response(NULL, 301, array('Location' => $returnURL));

    } else {
      $state = 'fail';
      $this->psh->setState($trxid, $state, $context);

      $comment = date('d.m.Y H:i:s').': 3D Secure Error '.$res->errorMSG;
      $fields  = array(
        'mcpay_authid'   => $check3dId,
        'mcpay_trxid'    => $trxid,
        'mcpay_comments' => $comment,
      );
      $this->psh->setCustomFields($orderId, $fields, $context);

      return new Response(NULL, 301, array('Location' => $errorURL));
    }
  }

  /**
   * handleNO3DAction
   *
   * @param $post
   * @param $context
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function handleNO3DAction($post, $context)
  {
    $crlf   = "\r\n";
    $orderId  = $post['orderid'];
    $trxid    = $post['trxid'];
    $returnURL = urldecode($post['returnURL']);
    $errorURL = urldecode($post['errorURL']);

    $state = 'paid';
    $this->psh->setState($trxid, $state, $context);

    $comment = date('d.m.Y H:i:s').': handleNO3D';
    $fields  = array(
      'mcpay_trxid'    => $trxid,
      'mcpay_comments' => $comment,
    );
    $this->psh->setCustomFields($orderId, $fields, $context);

    return new Response(NULL, 301, array('Location' => $returnURL));
  }

  /**
   * handleTESTAction
   *
   * @param $data
   * @param $context
   * @param $config
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function handleTESTAction($data, $context, $config)
  {
    $content = '<pre>';
    $content.= print_r($data,TRUE);
    $content.= print_r($config,TRUE);
    $content.= '</pre>';

    $trxid = $data['trxid'];
    $state = 'reopen';
    $this->psh->setState($trxid, $state, $context);

    return new Response($content);
  }

}